/*********************************************************************
 * Desktop Station joy R1 Sketch for Mr Push R2
 *
 * Copyright (C) 2016 Yaasan
 *
 */

// include the library code:
#include <stdint.h>
#include "Wire.h"
#include <EEPROM.h>
#include <avr/io.h>
#include <util/delay.h>
#include <avr/interrupt.h>
#include <avr/pgmspace.h>
#include "I2CLCDLib.h"
#include "Functions.h"
#include "ThrottleS88.h"
#include "TrackReporterS88_DS.h"


#define MAX_S88DECODER 4
#define RELPYERROR_300 "300 Command error"
#define RELPYERROR_301 "301 Syntax error"
#define RELPYERROR_302 "302 receive timeout"
#define RELPYERROR_303 "303 Unknown error"
#define RELPYERROR_NONE ""

#define THRESHOLD_EDC 186  // 1024d * (1 / 11) * 10V / 5V
#define THRESHOLD_OV  372  // 1024d * (1 / 11) * 20V / 5V

#define PIN_PWM_A 5
#define PIN_PWM_B 6


/* Decralation classes */
TrackReporterS88_DS reporter(MAX_S88DECODER);
ThrottleS88 DSJoy;

/* LCD functions */
I2CLCDLib LCD;

/* Parser */
String request;
String function;
word arguments[4];
word numOfArguments;
boolean result;

/* Button */
uint8_t gLastButton = 1;


/* analog Control */
uint8_t gAnaPower = 0;
uint16_t gAnaSpeed = 0;
uint8_t gAnaDir = 0;


//Task Schedule
unsigned long gPreviousL5 = 0; //   50ms interval
unsigned long gPreviousL8 = 0; // 1000ms interval
unsigned long gPreviousL9 = 0; // 2000ms interval



void printString(const char *s, int x, int y);
void DisplayTopLCD(word inAddress);
void TogglePower();

/***********************************************************************
* 
* Boot setup
* 
* 
************************************************************************/

void setup()
{
	DSJoy.ChangedPowerStatus = ChangedPower;
	DSJoy.ChangedTargetSpeed = ChangedLocoSpeed;
	DSJoy.ChangedTargetDir = ChangedLocoDir;
	DSJoy.ChangedTargetFunc = ChangedLocoFunc;
	DSJoy.ChangedTurnout = ChangedTurnout;
	
	//Pin mode
	pinMode(A3, INPUT);
	digitalWrite(A3, 1); //pullup
	pinMode(A0, OUTPUT);
	digitalWrite(A0, 0); //Reset High
	delay(50);
	digitalWrite(A0, 1); //Reset High
	delay(50);
	
	pinMode(7, OUTPUT); //RUN LED
	pinMode(PIN_PWM_A, OUTPUT);
	pinMode(PIN_PWM_B, OUTPUT);
	
	/* Clear data of the S88 decoder */
	reporter.refresh();
	
	/* LCD init */
	LCD.begin();

	LCD.lcd_setCursor(0, 0);
	LCD.lcd_printStr("DSjoy   ");
	LCD.lcd_setCursor(0, 1);
	LCD.lcd_printStr("Analogue"); 
	delay(50);

	/* Debug message for PC */
	//シリアル通信開始
	Serial.begin(115200);
	Serial.println("--------------------------------------");
	Serial.println("Desktop Station joy firmware r1test   ");
	Serial.println("--------------------------------------");
	Serial.println("100 Ready");
	
	//Reset task
	gPreviousL5 = millis();
	gPreviousL8 = millis();
	
	//Output Init
	setPWM();
	
	sei();    //ENABLE INTERRUPTION
	
}






void DisplayTopLCD(word inAddress)
{
	char aTempStr[16];
	byte aDcc = 0;
	
	LCD.lcd_setCursor(0, 0);
	sprintf(aTempStr, "SPD %3d%s", (gAnaSpeed * 100) >> 10, gAnaDir == 1 ? "R" : "F");
	LCD.lcd_printStr(aTempStr); 
	
}


void ChangedPower(const byte inPower)
{
	char aTempStr[16];
	
	gAnaPower = inPower;
	setPWM();
	
	//debug str
	Serial.print("Power:");	
	Serial.println(inPower);  
	
	
	LCD.lcd_setCursor(0, 1);
	sprintf(aTempStr, "Power%s", inPower == 1 ? "On " : "Off");
	LCD.lcd_printStr(aTempStr); 

}

void ChangedLocoSpeed(const word inAddr, const word inSpeed, const bool inUpdate)
{
	
	gAnaSpeed = inSpeed;
	setPWM();
	
	//debug str
	char aTempStr[16];
	sprintf(aTempStr, "spd:%04d",inSpeed); 
	printString(aTempStr,0,1);
	Serial.print("Addr:");
	Serial.print(inAddr);
	Serial.print(" speed:");
	Serial.println(inSpeed);
}

void ChangedLocoDir(const word inAddr, const byte inDir, const bool inUpdate)
{
	
	gAnaDir = inDir;
	setPWM();

	//debug str
	char aTempStr[16];
	sprintf(aTempStr, "dir:%04d",inDir); 
	printString(aTempStr,0,1);
	Serial.print("Addr:");
	Serial.print(inAddr);
	Serial.print(" direction:");
	Serial.println(inDir);
}


void ChangedLocoFunc(const word inAddr, const uint8_t inFuncNo, const uint8_t inFuncPower, const bool inUpdate)
{

}

void ChangedTurnout(const word inAddr, const byte inDir, const bool inUpdate)
{
	
	//DSCore.SetTurnout(ADDR_ACC_DCC + inAddr - 1, (byte)inDir);
  //debug str
  char aTempStr[16];
  sprintf(aTempStr, "Turn:%01d  ",inDir); 
  printString(aTempStr,0,1);
  Serial.print("Addr:");
  Serial.print(inAddr);
  Serial.print(" inDir:");
  Serial.println(inDir);  	
}




void loop()
{
	uint8_t aErrNo = 0;
	
	while(1)
	{
		aErrNo = 0;
		
		/* Reset watchdog timer. */
		//wdt_reset(); 
		
		//if( (millis() - gPreviousL5) >= 250)
    if( (millis() - gPreviousL5) >= 100)
		{
			
		
			/* S88  */
			reporter.refresh(MAX_S88DECODER);
			
			/* データ処理 */
			
			for( int i = 0; i < MAX_S88DECODER; i++)
			{
				uint16_t aData = (reporter.getByte(i * 2 + 1) << 8) + reporter.getByte(i * 2 + 0);
				if( (i == 0) && (aData != 0))
				{
					Serial.print("s88_raw:");
					Serial.println(aData,BIN);
				}
				if( aData != 0xFFFF)
				{
					/* S88データをジョイスティックプロファイルに読み替え処理 */
					DSJoy.SetData(i, aData);
				}
				else
				{
					break;
				}
			}
			//Reset task
			gPreviousL5 = millis();
		}
		
		if( (millis() - gPreviousL8) >= 100)
		{
			uint8_t aCurrentButton = digitalRead(A3);
			
			if( (gLastButton == 1) && ( aCurrentButton == 0))
			{
				/* Power toggle */
				
				TogglePower();
				
			}
			
			gLastButton = aCurrentButton;
			
			//Reset task
			gPreviousL8 = millis();
			
		}
		
		
		
		
		/* OC check */
		
		
		
		/* OV check */
		if( analogRead(A7) > THRESHOLD_OV )
		{
			aErrNo = 2;
		}
		/* LV check */
		if( analogRead(A7) < THRESHOLD_EDC )
		{
			aErrNo = 1;
			
		}		
		/* エラー処理 */
		if( aErrNo > 0)
		{
			if( gAnaPower == 1)
			{
				PowerOffByErr(aErrNo);
			}
		}
		
	}
}

void TogglePower()
{
	char aTempStr[16];
	uint8_t aPower = gAnaPower;
	
	if( aPower == 0)
	{
		aPower = 1;
	}
	else
	{
		aPower = 0;
	}
	
	LCD.lcd_setCursor(0, 0);
	sprintf(aTempStr, "Power%s", aPower == 1 ? "On " : "Off");
	LCD.lcd_printStr(aTempStr); 
	LCD.lcd_setCursor(0, 1);
	LCD.lcd_printStr("        "); 
	
	/* パワー切替 */
	gAnaPower = aPower;
	gAnaSpeed = 0;
	setPWM();
	
}

void PowerOffByErr(uint8_t inErrNo)
{
	LCD.lcd_setCursor(0, 0);
	LCD.lcd_printStr( "PowerOff"); 
	LCD.lcd_setCursor(0, 1);
	
	switch(inErrNo)
	{
	case 0:
		LCD.lcd_printStr("ERR: UNK"); 
		break;
	case 1:
		LCD.lcd_printStr("ERR: LV "); 
		break;
	case 2:
		LCD.lcd_printStr("ERR: OV "); 
		break;
	case 3:
		LCD.lcd_printStr("ERR: OV "); 
		break;
	}
	
	/* パワーオフ */
	gAnaPower = 0;
	setPWM();
}


void printString(const char *s, int x, int y)
{
	LCD.lcd_setCursor(x, y);
	LCD.lcd_printStr(s);
}

void clearDisplay()
{
	LCD.lcd_clear();           //clear the screen and set start position to top left corner

}


void setPWM()
{
	
	if( gAnaPower == 0)
	{
		analogWrite(PIN_PWM_A, 0);
		analogWrite(PIN_PWM_B, 0);
		digitalWrite(7, 1);
		return;
	}
	
	switch(gAnaDir)
	{
	case 2:
		analogWrite(PIN_PWM_A, 255 - (gAnaSpeed >> 2));
		analogWrite(PIN_PWM_B, 255);
		digitalWrite(7, 0);
		break;
		
	case 1:
		analogWrite(PIN_PWM_A, 255);
		analogWrite(PIN_PWM_B, 255 - (gAnaSpeed >> 2));
		digitalWrite(7, 0);
		break;
		
	default:
		analogWrite(PIN_PWM_A, 0);
		analogWrite(PIN_PWM_B, 0);
		digitalWrite(7, 1);
		break;
	}
	
}


